% This script should reproduce the results given in the following 
% published paper (minor changes are possible).
%
% Marc Castella, Selwa Rafi, Pierre Comon and Wojciech Pieczynski, 
% Separation of instantaneous mixtures of a particular set of dependent 
% sources using classical ICA methods. 
% EURASIP Journal on Advances in Signal Processing 2013, 2013:62.
%
% Author: Marc Castella
% Created: ??/12/2012
% Last modified: 23/10/2013
% Contact: marc.castella@telecom-sudparis.eu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% This work is licensed under a 
%%% Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.
%%% http://creativecommons.org/licenses/by-nc-sa/3.0/
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear; 

T = 2000;   % number of samples
NITER = 30; % number of ICE iterations

Ns = 2;     % number of sources
Netats = 2; % number of states of the hidden process 
pihat_Ini = ones(Netats,1)./Netats;

ESTIM_PARAM_R = true;   % choose whether parameters of hidden process should
                        % be estimated (default) or are known (in this case: 
                        % to be specified at initialization)

N_ICE = 1;      % number of samples for approximation of the conditional 
                %   expectation in ICE (denoted by K in (20))
lambda_ICE = 5; % lambda in the assumed ICE model (can be different from 
                % the value used in simulating the sources, case of
                % example 2, A5'')


p1 = 0.5;       %p1 = Prob(r=0) = Prob(s1 and s2 independent) 


% generating the auxiliary process r (in this program, r is iid)
r = rand(1,T)>p1;

%-- generating independent sources
stmp1 = generSources(Ns,T,'rand');
%-- generating dependent sources 
    %-- example 1 satisfying A5' (s1=epsilon*s2 with epsilon =+-1)
    tmp = generSources(1,T,'MoG',0.5);
    stmp2 = [tmp; tmp.*generSources(1,T,1)]; clear tmp;
    %-- example 2 satisfying A5'' (Laplace x Gaussianne along the 1st bisector
    % lambda = 20; sigma_l = sqrt(2*(1-1/lambda^2)); alpha_l = sqrt(3)*sigma_l;
    % tmp = [  sigma_l*randn(1,T); % alpha*2*(rand(1,T)-0.5); % 
    %     -1/lambda*log(rand(1,T)) + 1/lambda*log(rand(1,T)); ];
    %     rot = 1/sqrt(2)*[-1 -1; -1 1];
    % stmp2 = rot*tmp;
%-- generating dependent sources satisfying A3, A4, A5
%-----------------------------------------------------
s = ([1 1]'*(r==0)).*stmp1 + ([1 1]'*(r==1)).*stmp2;

%-- mixing the sources
A = randn(Ns); 
x = A*s;


%% -- ICA/ICE procedure
% initialization of the parameters
Ahat_Ini = aci_normalise(x); Bhat_Ini = pinv(Ahat_Ini);
pihat = pihat_Ini; Bhat = Bhat_Ini;

for ii = 1:NITER, % ICE iterations
    
    %%% --- calculate probability conditionnally to r: Pxr(i,t) = Prob(x(t)/r(t)=i-1)
    y = Bhat*x;

    % conditionnally to r=0 (independent sources)
    %%-- Prob(x(t)/r(t)=i-1) is assumed gaussian
         sigmatmp0 = 1;
         for t = 1:T,
             Pxr(1,t) = 1/(2*pi*sigmatmp0^2) * exp( - (y(:,t)'*y(:,t))/(2*sigmatmp0^2) );
         end;
   
    % conditionnally to r=1 (dependent sources)
    %%-- along the 2 bisectors, Gaussian x Laplace
    z = 1/sqrt(2)*[1 1;1 -1]*y;
    if ~ ( exist('sigmatmp1','var')&&exist('lambdatmp1','var') ),
        if exist('lambda','var'),
            lambdatmp1 = lambda; sigmatmp1 = sqrt(2*(1-1/lambdatmp1^2));
        elseif ~exist('lambda','var'),
            lambdatmp1 = lambda_ICE; sigmatmp1 = sqrt(2*(1-1/lambdatmp1^2));
        end;
    end;
    for t =1:T,
        Pxr(2,t) = ...
            1/2* 1/(sigmatmp1*sqrt(2*pi))*exp( -z(1,t)^2/(2*sigmatmp1^2) ) * lambdatmp1/2*exp(-lambdatmp1*abs(z(2,t))) + ...
            1/2* 1/(sigmatmp1*sqrt(2*pi))*exp( -z(2,t)^2/(2*sigmatmp1^2) ) * lambdatmp1/2*exp(-lambdatmp1*abs(z(1,t)));
    end; 

    Pconjointexr = diag(pihat)*Pxr;
    Pposteriorir = Pconjointexr./repmat( sum(Pconjointexr,1), 2,1);
    
    %--- re-estimation of the parameters ---
    if ESTIM_PARAM_R == true,
        pihat = 1/T*sum(Pposteriorir,2);
    else
    end;
    
    %--- simulating hidden variables according the the a posteriori distribution ---
    for n_ice = 1:N_ICE,
        U = rand;
        r_ice(n_ice,:) = double( rand(1,T)>Pposteriorir(1,:) );
        Ahat = aci_normalise( x(:, find(~r_ice(n_ice,:)) ) ); Bhattmp(:,:,n_ice) = pinv(Ahat);
    end;
   
    Bhat = mean(Bhattmp,3);
    
end; % ICE iterations

%-- final estimates --
G.ice(:,:) = Bhat*A;    % global separation matrix
y = Bhat*x;             % estimated sources
MSE.ice = elimamb_g3(y,s,1);    %MSE criterion
TxBonneSegm.ICE = ( sum(repmat(r,N_ICE,1)==r_ice,2)/T )';   % segmentation rate (as defined in eq. (29))

%% -- display results --
figure(1); plot(s(1, find(~r_ice(n_ice,:))), s(2, find(~r_ice(n_ice,:))), 'b.'); hold on; plot(s(1, find(r_ice(n_ice,:))), s(2, find(r_ice(n_ice,:))), 'g.'); hold off;
title('Segmentation result provided by r_{ice} on the original sources');
disp('--- Combined ICA/ICE results -----------');
    fprintf('Segmentation rate: %1.2f ---\n',TxBonneSegm.ICE);
    fprintf('Glogal mixing-separating matrix G (should be identity up to ambiguities)\n');
    disp(G.ice);
    fprintf('Separation criterion calculated on G:  %1.4f %1.4f\n',( 1 - max(G.ice.^2,[],2)./sum(G.ice.^2,2) )' )
    fprintf('MSE on each source'); disp(MSE.ice);

%% -- compare when ignoring dependence
[F] = aci_normalise(x); B = pinv(F); G.ign(:,:) = B*A; 
y = B*x;
MSE.ign = elimamb_g3(y,s,1);

x_tronc = x(:,find(~r));
    if isempty(x_tronc),
        MSE.con = nan(1,Ns);
    else,
        [F] = aci_normalise(x_tronc); B = pinv(F); G.con(:,:) = B*A;
        y = B*x;
        MSE.con = elimamb_g3(s,y,1);
    end;

disp('--- Results when ignoring dependence ---');
    fprintf('Glogal mixing-separating matrix G (should be identity up to ambiguities)\n');
    disp(G.ign);
    fprintf('Separation criterion calculated on G:  %1.4f %1.4f\n',( 1 - max(G.ign.^2,[],2)./sum(G.ign.^2,2) )' )    
    fprintf('MSE on each source'); disp(MSE.ign);


%% -- compare when knowing hidden process r    
disp('--- Results when r known ---------------');
    fprintf('Glogal mixing-separating matrix G (should be identity up to ambiguities)\n');
    disp(G.con);
    fprintf('Separation criterion calculated on G:  %1.4f %1.4f\n',( 1 - max(G.con.^2,[],2)./sum(G.con.^2,2) )'  )
    fprintf('MSE on each source'); disp(MSE.con);

disp('--- END ---');
    

