function [s,varargout] = generSources(n,Ne,choix_source,varargin);

% [s] = generSources(n,Ne,SourceChoice,varargin);
%
% random signal generator
% generSources generates source signals using Matlab random number 
% generators. The source signal caracteristics are defined with the input
% arguments SourceChoice (and possibly varargin).
%
% The genereted source signals are stationary (possibly non i.i.d.) and are
% normalized to unit power.
%
% Inputs:
% -------
% n = number of sources
% Ne = number of samples
% SourceChoice = 1 or 'BPSK' : i.i.d. binary +-1 with resp. probability 1/2
%                2 or 'rand' : i.i.d. uniformly distributed
%                3 or 'PAM4' : i.i.d. PAM4
%                4 or 'StVol': non i.i.d. stochastic volatility
%                   varargin = q: 
%                       q is the order of the MA auxiliary process
%                   Default value: q = 3
%                5 or 'ARCH': non i.i.d. ARCH
%                   varargin = q,totalphai: 
%                       q is the order of the ARCH process
%                       totalphai is the sum of the coefficients defining
%                       the recursive equation
%                   Default value: q = 3, totalphai = 0.9
%                6 or 'QAM4' : i.i.d. complex QAM4
%                7 or 'QAM16': i.i.d. complex QAM16
%                8 or 'CPM'  : non i.i.d. CPM (continuous phase modulation)
%                pseudo-symbols (valid when sampling at the symbol period)
%                   varargin = h:
%                       h is vector of length n with the respective 
%                       modulation indices of the CPM sources. If h is a 
%                       scalar, all sources have the same modulation index.
%                9 or 'Lapl' : i.i.d. double exponential (Laplace)
%		 10 or 'MoG' : mixture of two Gaussians 1/2(N(mu,sigma)+N(-mu,sigma))
%		    varargin = mu:
%			sigma is such that mu^2+sigma^2=1, (which ensures unit variance)
%
% Output:
% -------
% s = (n x Ne) matrix with Ne samples of the n signals
% 
% -------
% Author: Marc Castella
% Created: 24/04/2002
% Last modified: 23/10/2013
% Contact: marc.castella@telecom-sudparis.eu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% This work is licensed under a 
%%% Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.
%%% http://creativecommons.org/licenses/by-nc-sa/3.0/
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% modif 19/07/2002
% modif 23/07/2002 relier et ameliorer la generation des ARCH
% modif 16/10/2003 ajoute option 5 : cex
% modif 18/01/2006 supprime generation cex, ajoute QAM4/16 et CPM
% modif 08/07/2011 commentaires actualises
% TO DO: 
%                5 : cex  5) epsilon (cumulant en 0,0,0)
% case 5,    s = genecex(n,Ne,varargin{1});

if nargout>1,
    if choix_source~=8,
        error('Error using generSources: wrong number of output arguments');
    end;
end;

switch choix_source
case {1,'BPSK'} %binaires +-1
    if nargin ~= 3, error('Error using generSources: wrong number of input arguments'); end;
    s = sign(randn(n,Ne));
case {2,'rand'} %sources rand
    if nargin ~= 3, error('Error using generSources: wrong number of input arguments'); end;
    s = rand(n,Ne)-0.5;
case {3,'PAM4'} %sources PAM
    if nargin ~= 3, error('Error using generSources: wrong number of input arguments'); end;
    s = unidrnd(4,n,Ne)-2.5;
case {4,'StVol'} %sources volatilite stochastique
    if nargin == 4,
        q = varargin{1};
    else, % default value
        q = 3;
    end;
    theta_MA = [randn(n,q) ones(n,1)];
    epsilon = randn(n,Ne+q);
    a = zeros(n,Ne);
    for ii=1:Ne,
        a(:,ii) = diag(epsilon(:,ii:ii+q)*theta_MA');
    end;
    w = randn(n,Ne);
    s = a.*w;
    %clear  epsilon w a theta_MA
case {5,'ARCH'}
    if nargin == 4,
        q = varargin{1};
    elseif nargin == 5,
        q = varargin{1};
        totalphai = varargin{2};
    else % default value
        q = 3;
        totalphai = 0.9;
    end;
    alpha = abs(randn(n,q+1));
    for ii = 1:n,
        existmom4 = 2; nessai = 0;
        while existmom4>1 & nessai<10,
            nessai = nessai + 1;
            alpha(ii,1:q) = 1/sum(alpha(ii,1:q))*totalphai.*alpha(ii,1:q);
            
            %rajouter un test de la condition 3alpha'*inv(I-psi)*alpha < 1 :
            %permet de guarantir l'existence des moments d'ordre 4!!!
            a = fliplr(alpha(ii,1:q));
            % la matrice des alpha_{ii+j}
            psi1 = hankel([a(2:end) 0]);
            % la matrice des alpha_{ii-j}
            psi2 = toeplitz([0 a(1:end-1)]',[zeros(1,q)]);
            psi = psi2-psi1;
            existmom4 = 3*a*inv(eye(q)-psi)*a';
        end;
        if nessai == 10, 
            disp('generSources WARNING:');
            disp('Existence of 4th order moments not guaranteed');
        end;
    end;
    %if alpha(0) < 0.5, alpha(0)=alpha(0) + 0.5; end;
    epsilon = randn(n,Ne+q);
    s = epsilon(:,1:q); s2 = s.^2;
    for ii=1:Ne,
        hdemi = sqrt(diag(s2*alpha(:,1:q)')+alpha(:,q+1));
        s = [s epsilon(:,q+ii).*hdemi ];
        %s2 = [s2(:,1:q-1) s(:,q+ii).^2]; 
        s2 = [s2(:,2:q) s(:,q+ii).^2]; 
    end;
    s = s(:,q+1:Ne+q);
    %disp(alpha);
case {6,'QAM4'} % QAM4
    if nargin ~= 3, error('Error using generSources: wrong number of input arguments'); end;
    s = (unidrnd(2,n,Ne)-1.5) + i*(unidrnd(2,n,Ne)-1.5);
case {7,'QAM16'} % QAM16
    if nargin ~= 3, error('Error using generSources: wrong number of input arguments'); end;
    s = (unidrnd(4,n,Ne)-2.5) + i*(unidrnd(4,n,Ne)-2.5);
case {8,'CPM'} % CPM
    if nargin == 4,
        h = varargin{1};
    else,
        error('Error using generSources: wrong number of input arguments');
    end;
    a = sign(randn(n,Ne)); 
    varargout(1) = {a};
    s = diag(exp(i*pi*rand(1,n)))*exp(i*pi*diag(h)*cumsum(a,2));
case {9,'Lapl'} % double exponential (Laplace)
    if nargin ~= 3, error('Error using generSources: wrong number of input arguments'); end;    
    s = -1/sqrt(2)*log(rand(n,Ne)) + 1/sqrt(2)*log(rand(n,Ne));
case {10,'MoG'} % Mixture of two Gaussians
    if nargin == 3, 
        mu = 1/sqrt(2); sigma = 1/sqrt(2);
    elseif nargin == 4,
        mu = varargin{1}; if mu>=1, error('Error: parameter mu should be <1'); end;
        sigma = sqrt(1-mu^2);
    end;
    r = sign(randn(n,Ne));
    s = (r==1).*(mu+sigma*randn(n,Ne)) + (r==-1).*(-mu+sigma*randn(n,Ne));
end;

% normaliser variance unite
for ii=1:n,
    s(ii,:) = s(ii,:)/std(s(ii,:),1);
end;

